#include "artmap.h"

int main(int argc, char *argv[]) {
	if (argc == 2) {
		int option;
		sscanf (argv[1], "%d", &option);

		if (option == 1) {
			artmap *am = new artmap(2, 3);
			am->setNetworkType(artmap::FUZZY);
			float *in = new float(2);

			in[0] = 0.1f;	in[1] = 0.1f;	am->train (in, 0);
			in[0] = 0.9f;	in[1] = 0.6f;	am->train (in, 1);
			in[0] = 0.4f;	in[1] = 0.9f;	am->train (in, 2);

			for (float y = 1.0f; y >= 0.0f; y -= 0.05f) {
				for (float x = 0.0f; x <= 1.0f; x += 0.03f) {
					in[0] = x; in[1] = y;
					am->test (in);
					int result = am->getMaxOutputIndex();
					if (result == -1) cout << " ";
					else cout << result;
				}
				cout << "\n";
			}
		} else if (option == 2) {
			int numSamples = 16000;
			int numFeatures = 16;
			int numClasses = 26;

			artmap *am = new artmap(numFeatures, numClasses);

			ifstream *ifs = new ifstream ("\\d_Letters\\data16k.txt", ios::in);
			if (!(*ifs)) cout << "Failed to open letters data\n";

			int *sampleClasses = new int[numSamples];
			float *sampleData = new float [numSamples * numFeatures];

			string *line = new string();
			for (int i = 0; i < 10; i++) std::getline (*ifs, *line); // Get past the header

			char ts[128];
			timeStamp(ts); cout << "Reading data, t = " << ts << endl;
			int cl;
			float dataPt;
			for (int sampleIdx = 0; sampleIdx < numSamples; sampleIdx++) {
				*ifs >> cl;
				sampleClasses[sampleIdx] = cl;
				for (int featureIdx = 0; featureIdx < numFeatures; featureIdx++) {
					*ifs >> dataPt;
					sampleData[sampleIdx * numFeatures + featureIdx] = dataPt;
				}
			}
			timeStamp(ts); cout << "Done, t = " << ts << endl;

			timeStamp(ts); cout << "Training, t = " << ts << endl;
			for (int sampleIdx = 0; sampleIdx < numSamples; sampleIdx++) {
				am->train(&sampleData[sampleIdx * numFeatures], sampleClasses[sampleIdx]);
			}
			timeStamp(ts); cout << "Done, t = " << ts << endl;

			timeStamp(ts); cout << "Testing, t = " << ts << endl;
			int correct = 0;
			for (int sampleIdx = 0; sampleIdx < numSamples; sampleIdx++) {
				am->test(&sampleData[sampleIdx * numFeatures]);
				if (am->getMaxOutputIndex() == sampleClasses[sampleIdx]) correct++;
			}
			timeStamp(ts); cout << "Done, t = " << ts << endl;
			cout << "Correct = " << correct << endl;
			cout << "F2 nodes: " << am->getC() << "\n";

		}
		return 0;
	} else 
		cout << "Arguments: \n";
		cout << " 1 - Show decision surface with three training points\n";
	  cout << " 2 - Test timing over full letters data set (train & test with same 16K points)\n";
		return 1;
}